/*=============================================================================
XMOTO

This file is part of XMOTO.

XMOTO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

XMOTO is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with XMOTO; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
=============================================================================*/

_("First let's take a look at the user interface...");
_("In the upper left corner, you've got your timers");
_("The big one is your current time");
_("The small one to the right is your own best time\nfor the current level...");
_("... and the one to the left is the best of all\non this computer.");
_("Down here there's the minimap, which shows the level\naround you, so you "
  "know where you're heading.");
_("Now let's look at your controls.");
_("(You seem to be using a joystick, can't help with that :P)");
_(": Drive forward");
_(": Brake");
_(": Change direction");
_(": Rotate anti-clockwise");
_(": Rotate clockwise");
_("Okay. The red dot on your minimap is a strawberry.");
_("The primary objective of the game is for you\nto collect all of them, and "
  "then find the flower.");
_("The icon to the right of your time,\nindicates how many strawberries still "
  "left.");
_("Drive in its direction...");
_("When done, head for the flower.\nIt's the purple dot on your minimap.");
_("Welcome to X-Moto!");
_("This level is a short tutorial of\nhow you play the game.");
_("Please stay put and follow the instructions! :)");
_("Pick up the strawberry by touching it");
_("Touch it to finish the level. Good luck with the rest :)");
_("The first jump you're going to perform is very easy.");
_("Drive to the left, accelerate, and when you are in\nthe air, try to adjust "
  "your attitude with the left\nand right arrow keys, so you'll land on your "
  "wheels.");
_("The next jump is a bit trickier. You'll need all the\nspeed you can get to "
  "reach the other side...");
_("Now for the most difficult jump in this tutorial!");
_("This time, speed alone won't do the job...");
_("Here's the plan:\nAgain, you'll have to obtain maximum speed, but when\nyou "
  "reach the edge of the jump, pull the bike's front\nwheel up.");
_("That will cause the rear suspension to increase the\nlength of the "
  "jump.\nTry it...");
_("This tutorial will teach you the basics of jumping.");
_("Remember you can turn around your bike\nby pressing :");
_("That was pretty easy, right?");
_("Accelerate NOW! Obtain maximum speed, and try to\nkeep the front wheel at "
  "the ground...");
_("Nice jump there! :)");
_("Congrats, you're now a master jumper!");
_("The basic idea is that when driving on the rear wheel,\nyour head is "
  "lowered quite a lot...");
_("The difficult part is to keep the balance.\nWhen the distance you have to "
  "drive is quite short,\nyou don't need to keep the balance.");
_("Accelerate (slowly) to the right with the front wheel raising.\nJust before "
  "your head hit the block, make a quick left/right\ncombination, and you "
  "should be on the other side.\nTry it...");
_("Now you'll try a bit more difficult one...\nThis time keep your hands off "
  "the left and right arrow keys,\nthey're not delicate enough to help you "
  "keep balance.");
_("Instead regulate your angle by accelerating and braking.\nTry to drive a "
  "bit back and forth on your rear wheel,\nkeeping the bike as steady as "
  "possible...");
_("Try getting under the next obstacle when you feel you're ready.");
_("That will cause the rear suspension to increase the\nlength of the "
  "jump.\nTry it...");
_("This tutorial will teach you to rear-wheel through\nareas where the ceiling "
  "is low.\nBeware that this can be pretty difficult :)");
_("Not that difficult, right?");
_("Nice! Now try the next one, it's a bit more difficult...");
_("Good job! The next one is even more difficult...");
_("Now for the final one... It's not that long, but instead\nthere's even less "
  "room for your precious head. Good luck! :)");
