/*=============================================================================
XMOTO

This file is part of XMOTO.

XMOTO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

XMOTO is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with XMOTO; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
=============================================================================*/

#ifndef __STATESERVERCONSOLE_H__
#define __STATESERVERCONSOLE_H__

#include "../gui/basic/GUIConsole.h"
#include "StateMenu.h"

class UIRoot;
class UIConsole;

class StateServerConsole : public StateMenu, public UIConsoleHook {
public:
  StateServerConsole(bool drawStateBehind = true,
                     bool updateStatesBehind = false);
  virtual ~StateServerConsole();

  void enter();

  /* input */
  virtual void xmKey(InputEventType i_type, const XMKey &i_xmkey);
  void executeOneCommand(std::string cmd, std::string args);

  static void clean();

  /* console implementation */
  void exec(const std::string &i_cmd);
  void exit();

protected:
  virtual void checkEvents();

private:
  /* GUI */
  static UIRoot *m_sGUI;
  UIConsole *m_console;

  void createGUIIfNeeded();
};

#endif
